import 'dart:math';
import 'dart:typed_data';
import 'dart:convert';
import 'package:dvote/net/http.dart';

const IPFS_GATEWAY_LIST_URI =
    "https://ipfs.github.io/public-gateway-checker/gateways.json";

/// Launches 3 concurrent requests to well-known IPFS gateways and returns
/// the valid response of the first of them to reply.
/// Produces a time out after 20 seconds
Future<Uint8List> fetchIpfsHash(String hash) {
  return _getRandomIpfsURLs().then((gwList) {
    return Future.any(gwList
        .map((gwUrl) => httpGetWithTimeout(
            gwUrl.replaceFirst(RegExp(":hash\$"), hash),
            timeout: 15))
        .cast<Future<Uint8List>>()
        .toList());
  });
}

Future<List<String>> _getRandomIpfsURLs() {
  return httpGetStringWithTimeout(IPFS_GATEWAY_LIST_URI, timeout: 8)
      .then((gwResponse) {
    final List<String> gwList = jsonDecode(gwResponse);
    if (!(gwList is List)) throw Exception("Invalid gateway list response");

    return _shuffledList(gwList).sublist(0, 2);
  });
}

List<String> _shuffledList(List<String> items) {
  var random = Random();

  items.sort((_, __) => random.nextBool() ? 1 : -1);
  return items.whereType<String>().cast<String>().toList();
}
